'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var VCFVariant = function () {
  function VCFVariant(line) {
    var _this = this;

    var samples = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];
    var isSynth = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

    _classCallCheck(this, VCFVariant);

    this._line = line;
    this.isSynth = isSynth;

    var fields = this._line.split('\t', samples.length > 0 ? samples.length + 9 : 8);
    this.contig = fields[0]; // eslint-disable-line prefer-destructuring
    this.position = parseInt(fields[1], 10);
    var id = fields[2];
    this.id = id === '.' ? undefined : id.split(';');
    this.ref = fields[3].toUpperCase(); // eslint-disable-line prefer-destructuring
    this.alt = fields[4].split(',').map(function (allele) {
      return (/^[ACGTN]+$/i.test(allele) ? allele.toUpperCase() : allele
      );
    });

    var filter = fields[6];
    this.filter = filter === '.' || filter === '' ? undefined : filter.split(';');

    // Parse genotypes
    this._genotypes = new Map();
    for (var s = 0; s < samples.length; s += 1) {
      // GT must be the first field for each sample
      var endOfGT = fields[s + 9].indexOf(':');
      var GT = endOfGT === -1 ? fields[s + 9] : fields[s + 9].substring(0, endOfGT);

      // Translate alleles, while ignoring the distinction bewteen '/' and '|'
      var stringGT = GT.split(/[/|]/).sort() // Reference allele should be first
      .map(function (allele) {
        if (allele === '.') {
          return '.';
        }if (allele === '0') {
          return _this.ref;
        }
        return _this.alt[parseInt(allele, 10) - 1];
      }).join('/');
      this._genotypes.set(samples[s], stringGT);
    }
  }

  _createClass(VCFVariant, [{
    key: 'toString',
    value: function toString() {
      return this.contig + ':g.' + this.position + this.ref + '>' + this.alt.join(',');
    }
  }, {
    key: 'isBiAllelic',
    value: function isBiAllelic() {
      return this.alt.length === 1;
    }
  }, {
    key: 'isPASS',
    value: function isPASS() {
      return this.filter ? this.filter.length === 1 && this.filter[0] === 'PASS' : false;
    }
  }, {
    key: 'isFILTER',
    value: function isFILTER() {
      return this.filter ? this.filter.length >= 1 && this.filter[0] !== 'PASS' : false;
    }

    /**
     * If no sample is specified, return the 1st genotype
     */

  }, {
    key: 'genotype',
    value: function genotype(sample) {
      return sample === undefined ? this._genotypes.values().next().value : this._genotypes.get(sample);
    }
  }]);

  return VCFVariant;
}();

module.exports = VCFVariant;