'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var fs = require('fs');
var http = require('http');
var URL = require('url');
var AbstractFileReader = require('./AbstractFileReader');

function bufferToArrayBufferSlice(buffer, bytes) {
  var bytesToRead = bytes === undefined ? buffer.byteLength : bytes;
  return buffer.buffer.slice(buffer.byteOffset, buffer.byteOffset + bytesToRead);
}

var LocalFileReader = function (_AbstractFileReader) {
  _inherits(LocalFileReader, _AbstractFileReader);

  function LocalFileReader(path) {
    _classCallCheck(this, LocalFileReader);

    var _this = _possibleConstructorReturn(this, (LocalFileReader.__proto__ || Object.getPrototypeOf(LocalFileReader)).call(this));

    _this.handle = fs.openSync(path, 'r');
    _this.size = fs.statSync(path).size;
    return _this;
  }

  _createClass(LocalFileReader, [{
    key: 'bytes',
    value: function bytes() {
      var _this2 = this;

      var start = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
      var length = arguments[1];

      return new Promise(function (resolve, reject) {
        var lengthToRead = length === undefined ? _this2.size - start : length;
        var buffer = Buffer.alloc(lengthToRead);
        fs.read(_this2.handle, buffer, 0, lengthToRead, start, function (err, read, buf) {
          if (err) {
            reject(new Error(err));
          } else {
            resolve(bufferToArrayBufferSlice(buf, read));
          }
        });
      });
    }
  }]);

  return LocalFileReader;
}(AbstractFileReader);

var RemoteFileReader = function (_AbstractFileReader2) {
  _inherits(RemoteFileReader, _AbstractFileReader2);

  function RemoteFileReader(url) {
    _classCallCheck(this, RemoteFileReader);

    var _this3 = _possibleConstructorReturn(this, (RemoteFileReader.__proto__ || Object.getPrototypeOf(RemoteFileReader)).call(this));

    _this3.url = URL.parse(url);
    return _this3;
  }

  _createClass(RemoteFileReader, [{
    key: 'bytes',
    value: function bytes() {
      var start = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
      var length = arguments[1];

      var options = Object.assign({}, this.url, { method: 'GET' });

      if (start !== 0 || length !== undefined) {
        // Requesting only a portion of the file
        options.headers = {
          Range: length !== undefined ? 'bytes=' + start + '-' + (start + (length - 1)) : 'bytes=' + start + '-'
        };
      }

      return new Promise(function (resolve, reject) {
        var req = http.request(options, function (res) {
          var data = [];
          res.on('data', function (chunk) {
            data.push(chunk);
          }).on('end', function () {
            var buffer = Buffer.concat(data);
            resolve(bufferToArrayBufferSlice(buffer));
          });
        });
        req.on('error', function (e) {
          reject(new Error('RemoteFileRequest failed: ' + e.message));
        });
        req.end();
      });
    }
  }]);

  return RemoteFileReader;
}(AbstractFileReader);

module.exports = {
  LocalFileReader: LocalFileReader,
  RemoteFileReader: RemoteFileReader
};