'use strict';

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/* eslint-disable no-bitwise */
/* eslint new-cap: ["error", { "newIsCapExceptions": ["jBinary", "jDataView"] }] */

// This file contains code adapted from
// https://github.com/jsa-aerial/JS-Binary-VCF-Tabix
// The license for which is copied below:

// --------------------------------------------------------------------------//
//                                                                          //
//                        B I N A R Y - V C F                               //
//                                                                          //
//                                                                          //
// Copyright (c) 2014-2014 Trustees of Boston College                       //
//                                                                          //
// Permission is hereby granted, free of charge, to any person obtaining    //
// a copy of this software and associated documentation files (the          //
// "Software"), to deal in the Software without restriction, including      //
// without limitation the rights to use, copy, modify, merge, publish,      //
// distribute, sublicense, and/or sell copies of the Software, and to       //
// permit persons to whom the Software is furnished to do so, subject to    //
// the following conditions:                                                //
//                                                                          //
// The above copyright notice and this permission notice shall be           //
// included in all copies or substantial portions of the Software.          //
//                                                                          //
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,          //
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF       //
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND                    //
// NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE   //
// LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION   //
// OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION    //
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.          //
//                                                                          //
// Author: Jon Anthony                                                      //
//                                                                          //
// --------------------------------------------------------------------------//

// This file contains code adapted from
// https://github.com/hammerlab/pileup.js
// which uses the Apache 2.0 license included herein.


var jDataView = require('jdataview');
var jBinary = require('jbinary');
var pako = require('pako/lib/inflate');

var _require = require('text-encoding'),
    TextDecoder = _require.TextDecoder;

var defer = require('promise-defer');
var findLastIndex = require('lodash/findLastIndex');

var _require2 = require('../util/Errors'),
    ContigNotInIndexError = _require2.ContigNotInIndexError;

var AbstractFileReader = require('./AbstractFileReader');

function inflateOneGZipBlock(buffer, position) {
  var inflator = new pako.Inflate();
  inflator.push(new Uint8Array(buffer, position));
  return {
    err: inflator.err,
    msg: inflator.msg,
    buffer: inflator.result ? inflator.result.buffer : null,
    total_in: inflator.strm.total_in
  };
}

/**
 * Tabix files are compressed with BGZF, which consists of many concatenated
 * gzip'd blocks. These blocks must be decompressed separately.
 * @param lastBlockStart Stop decompression at this byte offset
 */
function inflateConcatenatedGZip(buffer, lastBlockStart) {
  var position = 0;
  var blocks = [];
  var maxOffset = lastBlockStart === undefined ? buffer.byteLength : lastBlockStart;
  do {
    var result = inflateOneGZipBlock(buffer, position);
    if (result.err) {
      throw new Error('Gzip error: ' + result.msg);
    }
    if (result.buffer) {
      blocks.push({
        offset: position,
        compressedLength: result.total_in,
        buffer: result.buffer
      });
    }
    position += result.total_in;
  } while (position <= maxOffset && position < buffer.byteLength);

  return blocks;
}

function concatArrayBuffers(buffers) {
  var totalBytes = buffers.map(function (b) {
    return b.byteLength;
  }).reduce(function (a, b) {
    return a + b;
  }, 0);
  var output = new Uint8Array(totalBytes);

  var position = 0;
  buffers.forEach(function (buffer) {
    output.set(new Uint8Array(buffer), position);
    position += buffer.byteLength;
  });

  return output.buffer;
}

/**
 * Inflate one or more gzip blocks in the buffer concatenating the results,
 * mirroring the behavior of gzip(1). Use lastBlockStart=0 to read a single
 * block.
 */
function inflateGZip(buffer, lastBlockStart) {
  return concatArrayBuffers(inflateConcatenatedGZip(buffer, lastBlockStart).map(function (x) {
    return x.buffer;
  }));
}

var VirtualOffset = function () {
  // Uncompressed offset

  function VirtualOffset(coffset, uoffset) {
    _classCallCheck(this, VirtualOffset);

    this.coffset = coffset;
    this.uoffset = uoffset;
  }

  // Compressed offset


  _createClass(VirtualOffset, [{
    key: 'compareTo',
    value: function compareTo(other) {
      return this.coffset - other.coffset || this.uoffset - other.uoffset;
    }
  }], [{
    key: 'fromBlob',
    value: function fromBlob(u8, offset) {
      var validOffset = offset || 0;
      var uoffset = u8[validOffset] + u8[validOffset + 1] * 256;
      var coffset = u8[validOffset + 2] + u8[validOffset + 3] * 256 + u8[validOffset + 4] * 65536 + u8[validOffset + 5] * 16777216 + u8[validOffset + 6] * 4294967296 + u8[validOffset + 7] * 1099511627776;
      return new VirtualOffset(coffset, uoffset);
    }
  }]);

  return VirtualOffset;
}();

// Tabix schema, as defined in http://samtools.github.io/hts-specs/tabix.pdf, adapted
// from https://github.com/jsa-aerial/JS-Binary-VCF-Tabix
var TABIX_FORMAT = {

  'jBinary.all': 'tabix',
  'jBinary.littleEndian': true,

  virtual_offset: jBinary.Template({ // eslint-disable-line new-cap
    baseType: 'uint64',
    read: function read() {
      var u64 = this.baseRead();
      return new VirtualOffset(
      // compressed offset
      u64.hi * 65536 + (u64.lo >>> 16),
      // uncompressed offset
      u64.lo & 0xffff);
    }
  }),

  header: {
    magic: ['const', ['string', 4], 'TBI\x01', true],
    n_ref: 'int32',
    format: 'int32',
    col_seq: 'int32',
    col_beg: 'int32',
    col_end: 'int32',
    meta: 'int32',
    skip: 'int32',
    l_nm: 'int32',
    names: ['string', function (context) {
      return context.l_nm;
    }]
  },

  chunk: {
    beg: 'virtual_offset',
    end: 'virtual_offset'
  },

  // Break chunk parsing apart as a performance optimization (adapted from pileup.js)
  // actual schema is:
  // chunks:    ['array', 'chunk', function(context){ return context.n_chunk; }]
  // intervals: ['array', 'virtual_offset', function(context) { return context.n_intv; }]

  chunks: ['array', 'chunk'],

  bin: {
    bin: 'uint32',
    n_chunk: 'int32',
    chunks: ['blob', function (context) {
      return 16 * context.n_chunk;
    }]
  },

  index: {
    n_bin: 'int32',
    bins: ['array', 'bin', function (context) {
      return context.n_bin;
    }],
    n_intv: 'int32',
    intervals: ['blob', function (context) {
      return 8 * context.n_intv;
    }]
  },

  tabix: {
    head: 'header',
    indexseq: ['array', 'index', function (context) {
      return context.head.n_ref;
    }]
  }

};

/**
 * Advance DataView by one BGZF block (without decompressing), returning
 * the compressed and uncompressed size for that block
 */
function advanceToEndOfBGZFBlock(view) {
  // Based on SAM specification: https://samtools.github.io/hts-specs/SAMv1.pdf
  view.skip(10); // Fixed header

  var bsize = void 0;
  var xlen = view.getUint16();

  var extraEnd = view.tell() + xlen;
  while (view.tell() < extraEnd) {
    var si1 = view.getUint8();
    var si2 = view.getUint8();
    if (si1 === 66 && si2 === 67) {
      view.getUint16(); // SLEN == 2
      bsize = view.getUint16();
      view.seek(extraEnd);
      break;
    } else {
      view.skip(view.getUint16()); // Skip extra field
    }
  }
  if (bsize === undefined) {
    throw new Error('Unable to determine block size');
  }

  view.skip(bsize - xlen - 19 + 4); // To start of ISIZE
  return { csize: bsize, usize: view.getUint32() };
}

/**
 * Advance DataView by one contig's Tabix index
 */
function advanceToEndOfIndex(view) {
  var numBins = view.getInt32();
  for (var b = 0; b < numBins; b += 1) {
    view.getUint32(); // bin ID
    var numChunks = view.getInt32();
    view.skip(numChunks * 16); // 16 bytes per chunk
  }
  view.skip(view.getInt32() * 8); // 8 bytes per interval element
}

function readChunks(buffer) {
  return new jBinary(buffer, TABIX_FORMAT).read('chunks');
}

function readInterval(buffer, index) {
  // Convert index to bytes
  return VirtualOffset.fromBlob(buffer, index * 8);
}

// Region-to-bins, as defined in http://samtools.github.io/hts-specs/tabix.pdf,
// adapted from https://github.com/hammerlab/pileup.js
function reg2bins(beg, end) {
  var k = void 0;
  var list = [];
  var incEnd = end - 1;
  list.push(0);
  for (k = 1 + (beg >> 26); k <= 1 + (incEnd >> 26); k += 1) {
    list.push(k);
  }for (k = 9 + (beg >> 23); k <= 9 + (incEnd >> 23); k += 1) {
    list.push(k);
  }for (k = 73 + (beg >> 20); k <= 73 + (incEnd >> 20); k += 1) {
    list.push(k);
  }for (k = 585 + (beg >> 17); k <= 585 + (incEnd >> 17); k += 1) {
    list.push(k);
  }for (k = 4681 + (beg >> 14); k <= 4681 + (incEnd >> 14); k += 1) {
    list.push(k);
  }return list;
}

function optimizeChunks(chunks, minimumOffset) {
  chunks.sort(function (l, r) {
    return l.beg.compareTo(r.beg) || l.end.compareTo(r.end);
  });

  var newChunks = [];
  chunks.forEach(function (chunk) {
    if (chunk.end.compareTo(minimumOffset) >= 0) {
      if (newChunks.length === 0) {
        newChunks.push(chunk);
      } else {
        // Merge overlapping or adjacent chunks
        var lastChunk = newChunks[newChunks.length - 1];
        if (chunk.beg.compareTo(lastChunk.end) > 0) {
          newChunks.push(chunk);
        } else {
          lastChunk.end = chunk.end;
        }
      }
    }
  });

  return newChunks;
}

function genericLineInRegion() {
  return true;
}

function vcfLineInRegion(line, ctg, pos, end) {
  var fields = line.split('\t', 8);
  if (fields.length < 8) {
    return false; // Malformed VCF line
  }

  if (fields[0] !== ctg) {
    // CHROM doesn't match
    return false;
  }

  var POS = parseInt(fields[1], 10);
  if (POS > end) {
    // POS beyond "end"
    return false;
  }

  // Determine END of VCF record, including END specified in INFO field
  var foundEND = /END=(\d+)/.exec(fields[7]);
  var END = foundEND ? parseInt(foundEND[1], 10) : POS + (fields[3].length - 1);
  if (END < pos) {
    return false;
  }

  return true;
}

var TabixIndexedFile = function () {
  function TabixIndexedFile(dataSource, indexSource) {
    _classCallCheck(this, TabixIndexedFile);

    this._source = dataSource;

    var indexBuffer = defer();
    var overlapFunction = defer();
    var commentCharacter = defer();

    this._indexBuffer = indexBuffer.promise;
    this._overlapFunction = overlapFunction.promise;
    this._commentCharacter = commentCharacter.promise;

    this._contigs = indexSource.bytes().then(function (buffer) {
      var uncompressedIndex = inflateGZip(buffer);
      indexBuffer.resolve(uncompressedIndex);

      var view = new jDataView(uncompressedIndex, 0, undefined, true /* little endian */);
      var parser = new jBinary(view, TABIX_FORMAT);

      // Parse header with metadata
      var head = parser.read(TABIX_FORMAT.header);

      // Set overlap function based on index header
      var format = head.format;

      switch (format) {
        case 2:
          overlapFunction.resolve(vcfLineInRegion);
          break;
        default:
          overlapFunction.resolve(genericLineInRegion);
          break;
      }

      // Extract comment character
      commentCharacter.resolve(String.fromCharCode(head.meta));

      // Compute contig indices to faciliate lazy parsing on indices (adapted from pileup.js)
      var names = head.names.replace(/\0+$/, '').split('\0');
      var contig2Index = new Map();

      for (var r = 0; r < head.n_ref; r += 1) {
        var contigBufferStart = view.tell();

        advanceToEndOfIndex(view);

        contig2Index.set(names[r], {
          bytes: [contigBufferStart, view.tell()],
          index: undefined
        });
      }

      return contig2Index;
    });
  }

  _createClass(TabixIndexedFile, [{
    key: '_chunksForInterval',
    value: function _chunksForInterval(ctg, pos, end) {
      var _this = this;

      return this._contigs.then(function (contigs) {
        var lazyIndex = contigs.get(ctg);
        if (!lazyIndex) {
          throw new ContigNotInIndexError('Unknown contig: ' + ctg);
        }

        if (!lazyIndex.index) {
          // Lazily parse index if needed
          lazyIndex.index = _this._indexBuffer.then(function (buffer) {
            var _lazyIndex$bytes = _slicedToArray(lazyIndex.bytes, 2),
                start = _lazyIndex$bytes[0],
                stop = _lazyIndex$bytes[1];

            var view = new jDataView(buffer, start, stop - start, true /* little endian */);
            var parser = new jBinary(view, TABIX_FORMAT);

            return parser.read(TABIX_FORMAT.index);
          });
        }
        return lazyIndex.index;
      }).then(function (index) {
        var bins = reg2bins(pos, end + 1);
        var chunks = index.bins.filter(function (b) {
          return bins.indexOf(b.bin) >= 0;
        }).map(function (b) {
          return readChunks(b.chunks);
        }).reduce(function (acc, cur) {
          return acc.concat(cur);
        }, []);

        // Apply linear index and other optimizations
        var minimumOffset = readInterval(index.intervals, Math.max(0, Math.floor(pos / 16384)));
        chunks = optimizeChunks(chunks, minimumOffset);

        return chunks;
      });
    }
  }, {
    key: '_fetchHeader',
    value: function _fetchHeader(offset) {
      // Read up to a single compressed block (no more than 64k)
      return Promise.all([this._source.bytes(offset, 65536), this._commentCharacter]).then(function (_ref) {
        var _ref2 = _slicedToArray(_ref, 2),
            buffer = _ref2[0],
            comment = _ref2[1];

        var uBuffer = inflateGZip(buffer, 0 /* Read single block */);
        var uView = new Uint8Array(uBuffer, 0, uBuffer.byteLength);

        var decoder = new TextDecoder('utf-8'); // VCF 4.3 allows UTF characters
        var lines = decoder.decode(uView).split(/\r?\n/); // VCF 4.3 allows LF or CRLF

        var last = findLastIndex(lines, function (line) {
          return line.startsWith(comment);
        });
        if (last === lines.length - 1) {
          throw new Error('Headers larger than single bgzip block not yet supported');
        }
        lines.splice(last + 1);

        return lines;
      });
    }
  }, {
    key: 'header',
    value: function header() {
      return this._fetchHeader(0);
    }
  }, {
    key: 'records',
    value: function records(ctg, pos, end) {
      var _this2 = this;

      var chunksPromise = this._chunksForInterval(ctg, pos, end);
      return Promise.all([chunksPromise, this._overlapFunction]).then(function (_ref3) {
        var _ref4 = _slicedToArray(_ref3, 2),
            chunks = _ref4[0],
            overlapFunction = _ref4[1];

        var decoder = new TextDecoder('utf-8'); // VCF 4.3 allows UTF characters

        // Read data for each chunk to produce array-of-array of decoded lines
        return Promise.all(chunks.map(function (chunk) {
          // At a minimum read at least one compressed block (which must be less than 64k)
          var cOffset = chunk.beg.coffset;
          var cBytes = chunk.end.coffset - chunk.beg.coffset + (chunk.end.uoffset > 0) ? 65536 : 0;

          return _this2._source.bytes(cOffset, cBytes).then(function (buffer) {
            var uOffset = chunk.beg.uoffset; // Start decoding at chunk's uncompressed offset
            var uBytes = chunk.end.uoffset - chunk.beg.uoffset;

            // Scan through compressed buffer to tally total uncompressed size
            var view = new jDataView(buffer, 0, undefined, true /* little endian */);
            while (view.tell() + cOffset < chunk.end.coffset) {
              uBytes += advanceToEndOfBGZFBlock(view).usize;
            }
            console.assert(view.tell() + cOffset === chunk.end.coffset);

            var uBuffer = inflateGZip(buffer, chunk.end.coffset /* Start of last block */);
            var uView = new Uint8Array(uBuffer, uOffset, uBytes);

            return decoder.decode(uView).split(/\r?\n/) // VCF 4.3 allows LF or CRLF
            .filter(function (line) {
              return line.length > 0 && overlapFunction(line, ctg, pos, end);
            });
          });
        })).then(function (lines) {
          return lines.reduce(function (acc, cur) {
            return acc.concat(cur);
          }, []);
        });
      });
    }
  }]);

  return TabixIndexedFile;
}();

module.exports = TabixIndexedFile;