'use strict';

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var findIndex = require('lodash/findIndex');
var last = require('lodash/last');
var isString = require('lodash/isString');
var defer = require('promise-defer');

var _require = require('../util/Errors'),
    ContigNotInIndexError = _require.ContigNotInIndexError;

var TabixIndexedFile = require('./TabixIndexedFile');
var Ref = require('../features/ReferenceGenome');
var VCFVariant = require('../features/VCFVariant');

var VCFSource = function () {
  function VCFSource(source, reference) {
    _classCallCheck(this, VCFSource);

    this._source = source;

    var referenceResolver = defer();
    if (reference !== undefined) {
      this._reference = Promise.resolve(reference);
    } else {
      // Will only be used if reference is not specified
      this._reference = referenceResolver.promise;
    }

    this._samples = this._source.header().then(function (headerLines) {
      if (!headerLines[0].startsWith('##fileformat=VCF')) {
        throw new Error('Source is not a valid VCF file');
      }

      // 1. Look for a reference line
      var refIdx = findIndex(headerLines, function (line) {
        return line.startsWith('##reference=');
      });
      if (refIdx !== -1) {
        // Do we know this reference file or string?
        var referenceField = headerLines[refIdx].substring(12);
        var referenceFrom = Ref.referenceFromFile(referenceField) || Ref.referenceFromShortName(referenceField);
        if (referenceFrom !== undefined) {
          referenceResolver.resolve(referenceFrom);
        }
      }

      // 2. Parse contig lines to infer reference
      var contigs = headerLines.filter(function (line) {
        return line.startsWith('##contig=');
      }).map(function (line) {
        return line.match(/ID=([^,>]+)/);
      }).filter(function (match) {
        return match && match.length === 2;
      }).map(function (match) {
        return match[1];
      });
      if (contigs.length > 0) {
        var _referenceFrom = Ref.referenceFromContigs(contigs);
        if (_referenceFrom !== undefined) {
          referenceResolver.resolve(_referenceFrom);
        }
      }

      // -OR- set hg19 as a default (will be a no-op if referenceResolver is already resolved)
      referenceResolver.resolve(Ref.hg19Reference);

      // Last line should be column labels
      var columns = last(headerLines).split('\t');
      if (columns[0] !== '#CHROM' || columns.length < 8) {
        throw new Error('Invalid column header line (#CHROM...)');
      }

      return columns.slice(9);
    });
  }

  _createClass(VCFSource, [{
    key: 'reference',
    value: function reference() {
      return this._reference;
    }
  }, {
    key: 'samples',
    value: function samples() {
      return this._samples;
    }

    /**
     * Normalize (array of) string regions to objects describing the sorted non-overlapping
     * regions.
     * @param  {[type]} regionOrRegions (Array of) contig:pos-end region strings
     * @return {Promise<Array<Region | Region>>} (Array of) region object with ctg, pos and end fields
     */

  }, {
    key: 'normalizeRegions',
    value: function normalizeRegions(regionOrRegions) {
      var _this = this;

      if (Array.isArray(regionOrRegions)) {
        return Promise.all(regionOrRegions.map(function (region) {
          return _this.normalizeRegions(region);
        })).then(function (regions) {
          return Promise.all([regions, _this._reference]);
        }).then(function (_ref) {
          var _ref2 = _slicedToArray(_ref, 2),
              regions = _ref2[0],
              reference = _ref2[1];

          regions.sort(function (aRegion, bRegion) {
            // eslint-disable-line arrow-body-style
            // Sort in reference order
            return reference.compareContig(aRegion.ctg, bRegion.ctg) || aRegion.pos - bRegion.pos || aRegion.end - bRegion.end;
          });
          // Merge overlapping regions
          return regions.reduce(function (prev, curr) {
            if (prev.length) {
              var tail = prev[prev.length - 1];
              if (curr.ctg === tail.ctg && curr.pos >= tail.pos && curr.pos <= tail.end) {
                tail.end = Math.max(tail.end, curr.end);
                return prev;
              }
            }
            prev.push(curr);
            return prev;
          }, []);
        });
      }if (isString(regionOrRegions)) {
        var _regionOrRegions$spli = regionOrRegions.split(/[:-]/, 3),
            _regionOrRegions$spli2 = _slicedToArray(_regionOrRegions$spli, 3),
            _ctg = _regionOrRegions$spli2[0],
            _pos = _regionOrRegions$spli2[1],
            _end = _regionOrRegions$spli2[2];

        return this._reference.then(function (ref) {
          return ref.normalizeContig(_ctg);
        }).then(function (normCtg) {
          return { ctg: normCtg, pos: parseInt(_pos, 10), end: parseInt(_end || _pos, 10) };
        });
      }
      throw new Error('Invalid region(s)');
    }

    /**
     * Query for variants overlapping genomic region
     * @param  {string} ctg Contig
     * @param  {number} pos Inclusive start of genomic region
     * @param  {number} end Inclusive end of genomic region
     * @return {Promise<Array<VCFVariant>>}     Array of VCFVariants overlapping region
     */

  }, {
    key: 'variants',
    value: function variants(ctg, pos, end) {
      var _this2 = this;

      var queryResults = this._reference.then(function (ref) {
        return ref.normalizeContig(ctg);
      }).then(function (normalizedCtg) {
        return _this2._source.records(normalizedCtg, pos, end);
      });

      return Promise.all([queryResults, this._samples]).then(function (_ref3) {
        var _ref4 = _slicedToArray(_ref3, 2),
            records = _ref4[0],
            samples = _ref4[1];

        return records.map(function (record) {
          return new VCFVariant(record, samples);
        });
      }, function (err) {
        if (err instanceof ContigNotInIndexError) {
          return [];
        }
        throw err;
      });
    }
  }, {
    key: '_synthVariant',
    value: function _synthVariant(ctg, pos, ref, alt) {
      return Promise.all([this._reference, this._samples]).then(function (_ref5) {
        var _ref6 = _slicedToArray(_ref5, 2),
            reference = _ref6[0],
            samples = _ref6[1];

        var synthRecord = reference.normalizeContig(ctg) + '\t' + pos + '\t.\t' + ref + '\t' + alt + '\t.\t.\t.';
        if (samples.length > 0) {
          synthRecord += '\tGT' + '\t0/0'.repeat(samples.length);
        }
        return new VCFVariant(synthRecord, samples, true /* isSynth */);
      });
    }

    /**
     * Query for single variant
     * @param  {string} ctg          Contig
     * @param  {number} pos          VCF position
     * @param  {string} ref          Reference allele
     * @param  {string} alt          Alternate allele
     * @param  {boolean} assumeRefRef If variant not found, synthesize variant with REF/REF genotype
     * @return {Promise<VCFVariant>}  Found (or synthetic) variant or undefined if
     * assumeRefRef is false and variant is not found
     */

  }, {
    key: 'variant',
    value: function variant(ctg, pos, ref, alt) {
      var _this3 = this;

      var assumeRefRef = arguments.length > 4 && arguments[4] !== undefined ? arguments[4] : false;

      return this.variants(ctg, pos, pos).then(function (variants) {
        // Filter for exact position and allele match, if none found and assumeRefRef
        // is true, synthesize a variant with a Ref/Ref genotype
        var foundVariant = variants.filter(function (variant) {
          return variant.ref === ref && variant.alt.indexOf(alt) !== -1;
        }).shift();
        if (!foundVariant && assumeRefRef) {
          return _this3._synthVariant(ctg, pos, ref, alt);
        }
        return foundVariant;
      });
    }
  }]);

  return VCFSource;
}();

module.exports = VCFSource;